# -*- encoding: utf-8 -*-

import pytest
import os

from os.path import expanduser, exists

from lib.disk import Disk

class TestDisk:
    def test_free_disk_space(self):
        assert Disk.free_disk_space("/etca") == None
        assert type(Disk.free_disk_space(expanduser("~"))) == int

    def test_path_writeble(self):
        assert Disk.path_writeble("/etca") == False
        assert Disk.path_writeble("/etc/passwd") == False
        assert Disk.path_writeble(expanduser("~")) == True

    def test_sizeof_fmt(self):
        assert Disk.sizeof_fmt(2691532) == "2.6 MiB"

    def test_get_su_created_folders(self):
        zero_su_folders = len(Disk.get_su_created_folders("/etc"))
        one_folder = len(Disk.get_su_created_folders("/home/meise/tmp"))
        assert zero_su_folders == 0
        assert one_folder == 1

    def test_is_su_folder(self):
        assert Disk.is_su_folder("00001-868e6-2e6-1m-home_matic") == True
        assert Disk.is_su_folder("foobar") == False

    def test_last_used_id(self):
        folders = ["00001-868", "00002-433", "00009-868-2e6-1m-home_matic"]
        assert Disk.last_used_id(folders) == 9

    def test_create_folder(self):
        path = r"/tmp/su_test_folder"
        Disk.create_folder(path)
        exists = os.path.exists(path)

        assert exists == True

        if exists:
            os.rmdir(path)

    def test_create_new_su_id(self):
        assert Disk.create_new_su_id(9) == "00009"
        assert Disk.create_new_su_id(1337) == "01337"

    def test_create_capture_folder(self):
        new_path = Disk.create_capture_folder("/home/meise/tmp/", "2.4e6",
                                              "868e6", "HomeMatic test 1")

        assert new_path == "/home/meise/tmp/00002-868e6-2.4e6-HomeMatic_test_1"
